import os
import json
import pytest

from barbell2 import CastorExportClient


HOME_DIR = os.environ['HOME']
EXPORT_FILE_DIR = os.path.join(HOME_DIR, 'data/surfdrive/projects/20210120_castorexportclient_test')
EXPORT_FILE_PATH = os.path.join(EXPORT_FILE_DIR, 'ESPRESSO_v2.0_DPCA_excel_export_20210120104629.xlsx')
OUTPUT_FILE_PATH = os.path.join(EXPORT_FILE_DIR, 'saved.csv')

# Requirements specification:
# https://docs.google.com/document/d/1cEtidByu0D2liaHwLqMXwiturqIICiWTa8Ta7NepK_U/edit


@pytest.fixture(scope="session", autouse=True)
def client():
    print('Creating client instance and loading data...')
    client = CastorExportClient(show_params=False)
    client.load_data(EXPORT_FILE_PATH)
    return client


# def test_query(client):
#     print(client.query('dpca_gebjaar > 1945')['dpca_gebjaar'])
#
# def test_view_weird_comorbidities(client):
#     d = client.data
#     for idx, row in d.iterrows():
#         x = row['dpca_comorb']
#         try:
#             int(x)
#         except ValueError:
#             print('{}: {}'.format(row['dpca_idcode'], x))
#
#
# def test_find_option_group_from_option_name(client):
#     option_groups = client.find_option_group('Chemoradio')
#     key = 'dpca_adjuvant'
#     assert len(option_groups.keys()) == 1
#     assert key in option_groups.keys()
#     assert option_groups[key] == [
#         (0, 'No'),
#         (1, 'Chemoradiotherapy'),
#         (2, 'Chemotherapy'),
#         (3, 'Radiotherapy'),
#         (7, 'Other'),
#         (9, 'Unknown')
#     ]
#
#
# def test_find_variable(client):
#     result = client.find_variable('gender')
#     print(result)
#
#
# def test_find_values(client):
#     result = client.find_values('dpca_comorb')
#     for item in result.iteritems():
#         print(item)
#
#
# def test_find_missing(client):
#     client.find_missing(in_column='dpca_datok', show_columns=['dpca_idcode', 'dpca_geslacht'])
#
#
# def test_find_duplicate_records(client):
#     duplicates = client.find_duplicate_records(columns=['dpca_idcode', 'dpca_datok'])
#     for k, v in duplicates.items():
#         print('{}: {}'.format(k, v))



import pydicom

from barbell2.utils import apply_window


class Dicom2Numpy:

    def __init__(self, dcm_file_path):
        self.dcm_file_path = dcm_file_path
        self.npy_array = None
        self.window = None
        self.normalize_enabled = True

    def set_window(self, window):
        self.window = window

    def set_normalize_enabled(self, normalize_enabled):
        self.normalize_enabled = normalize_enabled

    def is_normalize_enabled(self):
        return self.normalize_enabled

    def execute(self):
        self.npy_array = None
        p = pydicom.dcmread(self.dcm_file_path)
        pixels = p.pixel_array
        self.npy_array = pixels.reshape(p.Rows, p.Columns)
        if self.is_normalize_enabled():
            b = p.RescaleIntercept
            m = p.RescaleSlope
            self.npy_array = m * self.npy_array + b
        if self.window is not None:
            self.npy_array = apply_window(self.npy_array, self.window)
        return self.npy_array


if __name__ == '__main__':
    from npy2png import Numpy2Png
    d2n = Dicom2Numpy('/Volumes/GoogleDrive/My Drive/documents/projects/hpb/metabolicimaging/bodycomposition/20211109_demo_glasgow/pred/pancreas/4.dcm')
    d2n.set_window((400, 50))
    n2p = Numpy2Png(d2n.execute())
    n2p.set_output_dir('/Volumes/GoogleDrive/My Drive/documents/projects/hpb/metabolicimaging/bodycomposition/20211109_demo_glasgow/pred/pancreas/')
    n2p.execute()



import os
import numpy as np
import matplotlib.pyplot as plt

from barbell2.utils import apply_color_map, get_alberta_color_map


class Numpy2Png:

    def __init__(self, npy_array_or_file_path):
        self.npy_array_or_file_path = npy_array_or_file_path
        self.png_file_name = 'npy_array.png'
        self.png_file_path = None
        self.png_figure_size = (10, 10)
        self.color_map = None
        self.output_dir = '.'
        self.window = [400, 50]

    def set_png_file_name(self, png_file_name):
        self.png_file_name = png_file_name

    def set_png_figure_size(self, png_figure_size):
        self.png_figure_size = png_figure_size

    def set_output_dir(self, output_dir):
        self.output_dir = output_dir

    def set_color_map(self, color_map):
        if isinstance(color_map, str) and color_map == 'alberta':
            self.color_map = get_alberta_color_map()
        else:
            self.color_map = color_map

    def set_window(self, window):
        self.window = window

    def execute(self):
        if isinstance(self.npy_array_or_file_path, str):
            npy_array = np.load(self.npy_array_or_file_path)
        else:
            npy_array = self.npy_array_or_file_path
        if self.color_map is not None:
            npy_array = apply_color_map(npy_array, self.color_map)
        fig = plt.figure(figsize=self.png_figure_size)
        ax = fig.add_subplot(1, 1, 1)
        if self.color_map is not None:
            plt.imshow(npy_array)
        else:
            plt.imshow(npy_array, cmap='gray')
        ax.axis('off')
        self.png_file_path = os.path.join(self.output_dir, self.png_file_name)
        plt.savefig(self.png_file_path, bbox_inches='tight')
        plt.close('all')


if __name__ == '__main__':
    # pixels = np.load('/Users/Ralph/Desktop/output-segmentl3/1_pred.npy')
    n2p = Numpy2Png('/Users/Ralph/Desktop/output-segmentl3/1_pred.npy')
    n2p.set_color_map('alberta')
    n2p.set_output_dir('/Users/Ralph/Desktop')
    n2p.execute()



import os
import pydicom
import numpy as np

from .utils import get_alberta_color_map, create_fake_dicom


class Numpy2Dicom:

    def __init__(self, npy_array_or_file_path, dcm_file_path):
        self.npy_array_or_file_path = npy_array_or_file_path
        self.dcm_file_path = dcm_file_path
        self.color_map = get_alberta_color_map()
        self.npy_dcm_file_name = 'npy_array.dcm'
        self.npy_dcm_file_path = None
        self.output_dir = '.'

    def set_color_map(self, color_map):
        self.color_map = color_map

    def set_output_dir(self, output_dir):
        self.output_dir = output_dir

    def set_npy_dcm_file_name(self, npy_dcm_file_name):
        self.npy_dcm_file_name = npy_dcm_file_name

    def execute(self):
        if isinstance(self.npy_array_or_file_path, str):
            npy_array = np.load(self.npy_array_or_file_path)
        else:
            npy_array = self.npy_array_or_file_path
        p = pydicom.dcmread(self.dcm_file_path)
        if p.file_meta.TransferSyntaxUID.is_compressed:
            p.decompress()
        p_new = create_fake_dicom(npy_array, p)
        self.npy_dcm_file_path = os.path.join(self.output_dir, self.npy_dcm_file_name)
        p_new.save_as(self.npy_dcm_file_path)
